unit Sprite;

interface

uses
  SysUtils, WinTypes, WinProcs, Classes, Graphics, Controls, WinGPnt;

type

  TSprite = class( TObject )
  private
     nCycle: word;
     nDX, nDY: integer;
     nIncX, nIncY: integer;
     nError: integer;
     nInc: byte;
     w: TWinGPaintBox;
  protected
     ptDestination: TPoint;
     nSpeed: byte;
     procedure SetDestination( const pt: TPoint );
     procedure SetSpeed( const n: byte );
  public
     nPriority: integer;
     ptPosition: TPoint;
     constructor Create;
     procedure RegisterSprite( winGPnt: TWinGPaintBox );
     procedure UnRegisterSprite;
     procedure Move; virtual;
     procedure Render( winGPnt: TWinGPaintBox ); virtual; abstract;
     property Destination: TPoint read ptDestination write SetDestination;
     property Position: TPoint read ptPosition write ptPosition;
     property Speed: byte read nSpeed write SetSpeed;
  end;

implementation

constructor TSprite.Create;
begin
  w := nil;
  nPriority := 1;
  ptPosition := Point( 0, 0 );
  SetSpeed( 20 );
  SetDestination( Point( 0, 0 ) );
end;

procedure TSprite.RegisterSprite( winGPnt: TWinGPaintBox );
var
  nIndex, i: integer;
begin
  nIndex := -1;
  for i := 0 to winGPnt.lstSprites.Count - 1 do
     if TSprite( winGPnt.lstSprites[i] ).nPriority >= nPriority then
        begin
           nIndex := i;
           Break;
        end;
  if nIndex = -1 then
     winGPnt.lstSprites.Add( self )
  else
     winGPnt.lstSprites.Insert( nIndex, self );
end;

procedure TSprite.UnRegisterSprite;
var
  i: integer;
begin
  if w <> nil then
     begin
        for i := 0 to w.lstSprites.Count - 1 do
           if TSprite( w.lstSprites[i] ) = self then
              begin
                 w.lstSprites.Delete( i );
                 Break;
              end;
     end;
  Free;
end;

procedure TSprite.Move;
var
  bMove: boolean;
begin
  bMove := TRUE;
  if nIncX > 0 then
     begin
        if ptPosition.X >= ptDestination.X then
           bMove := FALSE;
     end
  else
     begin
        if ptPosition.X <= ptDestination.X then
           bMove := FALSE;
     end;
  if nIncY > 0 then
     begin
        if ptPosition.Y >= ptDestination.Y then
           bMove := FALSE;
     end
  else
     begin
        if ptPosition.Y <= ptDestination.Y then
           bMove := FALSE;
     end;
  if bMove then
     begin
        Inc( nCycle );
        if nCycle >= nSpeed then
           begin
              nCycle := 0;
              if nDX > nDY then
                 begin
                    Inc( nError, nDY );
                    if nError > nDX then
                       begin
                          Dec( nError, nDX );
                          Inc( ptPosition.Y, nIncY );
                       end;
                    Inc( ptPosition.X, nIncX );
                 end
              else
                 begin
                    Inc( nError, nDX );
                    if nError > 0 then
                       begin
                          Dec( nError, nDY );
                          Inc( ptPosition.X, nIncX );
                       end;
                    Inc( ptPosition.Y, nIncY );
                 end;
           end;
     end;
end;

procedure TSprite.SetDestination( const pt: TPoint );
begin
  nError := 0;
  ptDestination := pt;
  nDX := ptDestination.X - ptPosition.X;
  nDY := ptDestination.Y - ptPosition.Y;
  if nDX >= 0 then
     nIncX := nInc
  else
     begin
        nIncX := -nInc;
        nDX := -nDX;
     end;
  if nDY >= 0 then
     nIncY := nInc
  else
     begin
        nIncY := -nInc;
        nDY := -nDY;
     end;
end;

procedure TSprite.SetSpeed( const n: byte );
begin
  nSpeed := 0;
  if n < 7 then
     nInc := 3
  else if n < 15 then
     nInc := 2
  else
     begin
        nInc := 1;
        nSpeed := n - 15;
     end;
end;

end.
